@@ -1,7 +1,18 @@
 //-----------------------------------------------------------------------------
-// This code is licensed to you under the terms of the GNU GPL, version 2 or,
-// at your option, any later version. See the LICENSE.txt file for the text of
-// the license.
+// Copyright (C) Bogiton 2018
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
+//
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
 //-----------------------------------------------------------------------------
 // main code for standalone HF Sniff (and ULC/NTAG/ULEV1 pwd storing)
 //-----------------------------------------------------------------------------
@@ -102,7 +113,7 @@ static void RAMFUNC SniffAndStore(uint8_t param) {
     SpinDelay(50);
 
     // loop and listen
-    while (!BUTTON_PRESS()) {
+    while (BUTTON_PRESS() == false) {
         WDT_HIT();
         LED_A_ON();
 
@@ -225,7 +236,7 @@ static void RAMFUNC SniffAndStore(uint8_t param) {
     }
 
     if (g_dbglevel > 1)
-        Dbprintf("[!] Wrote %u Authentification attempts into logfile", auth_attempts);
+        Dbprintf("[!] Wrote %u Authentication attempts into logfile", auth_attempts);
 
     SpinErr(LED_A, 200, 5);
     SpinDelay(100);
