@@ -40,24 +40,34 @@ void CodeIso15693AsTag(const uint8_t *cmd, size_t len);
 
 void TransmitTo15693Reader(const uint8_t *cmd, size_t len, uint32_t *start_time, uint32_t slot_time, bool slow);
 int GetIso15693CommandFromReader(uint8_t *received, size_t max_len, uint32_t *eof_time);
-void TransmitTo15693Tag(const uint8_t *cmd, int len, uint32_t *start_time);
-int GetIso15693AnswerFromTag(uint8_t *response, uint16_t max_len, uint16_t timeout, uint32_t *eof_time);
+void TransmitTo15693Tag(const uint8_t *cmd, int len, uint32_t *start_time, bool shallow_mod);
+int GetIso15693AnswerFromTag(uint8_t *response, uint16_t max_len, uint16_t timeout, uint32_t *eof_time, bool fsk, bool recv_speed, uint16_t *resp_len);
 
 //void RecordRawAdcSamplesIso15693(void);
 void AcquireRawAdcSamplesIso15693(void);
-void ReaderIso15693(uint32_t parameter, iso15_card_select_t *p_card); // Simulate an ISO15693 reader - greg
-void SimTagIso15693(uint8_t *uid); // simulate an ISO15693 tag - greg
-void BruteforceIso15693Afi(uint32_t speed); // find an AFI of a tag - atrox
-void DirectTag15693Command(uint32_t datalen, uint32_t speed, uint32_t recv, uint8_t *data); // send arbitrary commands from CLI - atrox
+void ReaderIso15693(iso15_card_select_t *p_card); // ISO15693 reader
+void EmlClearIso15693(void);
+void EmlSetMemIso15693(uint8_t count, uint8_t *data, uint32_t offset);
+void EmlGetMemIso15693(uint8_t count, uint8_t *output, uint32_t offset);
+void SimTagIso15693(uint8_t *uid, uint8_t block_size); // simulate an ISO15693 tag
+void BruteforceIso15693Afi(uint32_t speed); // find an AFI of a tag
+void DirectTag15693Command(uint32_t datalen, uint32_t speed, uint32_t recv, uint8_t *data); // send arbitrary commands from CLI
 
-void SniffIso15693(uint8_t jam_search_len, uint8_t *jam_search_string);
+void SniffIso15693(uint8_t jam_search_len, uint8_t *jam_search_string, bool iclass);
 
 int SendDataTag(uint8_t *send, int sendlen, bool init, bool speed_fast, uint8_t *recv,
-                uint16_t max_recv_len, uint32_t start_time, uint16_t timeout, uint32_t *eof_time);
+                uint16_t max_recv_len, uint32_t start_time, uint16_t timeout, uint32_t *eof_time, uint16_t *resp_len);
 
-int SendDataTagEOF(uint8_t *recv, uint16_t max_recv_len, uint32_t start_time, uint16_t timeout, uint32_t *eof_time);
+int SendDataTagEOF(uint8_t *recv, uint16_t max_recv_len, uint32_t start_time, uint16_t timeout, uint32_t *eof_time, bool fsk, bool recv_speed, uint16_t *resp_len);
 
 void SetTag15693Uid(const uint8_t *uid);
 
-void DisablePrivacySlixLIso15693(uint8_t *password);
+void WritePasswordSlixIso15693(uint8_t *old_password, uint8_t *new_password, uint8_t pwd_id);
+void DisablePrivacySlixIso15693(uint8_t *password);
+void EnablePrivacySlixIso15693(uint8_t *password);
+void DisableEAS_AFISlixIso15693(uint8_t *password, bool usepwd);
+void EnableEAS_AFISlixIso15693(uint8_t *password, bool usepwd);
+void PassProtextEASSlixIso15693(uint8_t *password);
+void PassProtectAFISlixIso15693(uint8_t *password);
+void WriteAFIIso15693(uint8_t *password, bool usepwd, uint8_t *uid, bool use_uid, uint8_t afi);
 #endif
