@@ -29,6 +29,7 @@
 #include "cmdhficlass.h"  // pagemap
 #include "protocols.h"    // iclass defines
 #include "cmdhftopaz.h"   // TOPAZ defines
+#include "mifare/mifaredefault.h"     // MFP / AES defines
 
 #ifdef _WIN32
 #include "scandir.h"
@@ -85,6 +86,8 @@ DumpFileType_t getfiletype(const char *filename) {
             o = JSON;
         } else if (str_endswith(s, "dic")) {
             o = DICTIONARY;
+        } else if (str_endswith(s, "mct")) {
+            o = MCT;
         } else {
             // mfd, trc, trace is binary
             o = BIN;
@@ -112,27 +115,6 @@ int fileExists(const char *filename) {
     return result == 0;
 }
 
-/**
- * @brief checks if path is file.
- * @param filename
- * @return
- */
-/*
-static bool is_regular_file(const char *filename) {
-#ifdef _WIN32
-    struct _stat st;
-    if (_stat(filename, &st) == -1)
-        return false;
-#else
-    struct stat st;
-//    stat(filename, &st);
-    if (lstat(filename, &st) == -1)
-        return false;
-#endif
-    return S_ISREG(st.st_mode) != 0;
-}
-*/
-
 /**
  * @brief checks if path is directory.
  * @param filename
@@ -152,68 +134,6 @@ static bool is_directory(const char *filename) {
     return S_ISDIR(st.st_mode) != 0;
 }
 
-/**
- * @brief create a new directory.
- * @param dirname
- * @return
- */
-// Not used...
-/*
-#ifdef _WIN32
-#define make_dir(a) _mkdir(a)
-#else
-#define make_dir(a) mkdir(a,0755) //note 0755 MUST have leading 0 for octal linux file permissions
-#endif
-bool create_path(const char *dirname) {
-
-    if (dirname == NULL) // nothing to do
-        return false;
-
-    if ((strlen(dirname) == 1) && (dirname[0] == '/'))
-        return true;
-
-    if ((strlen(dirname) == 2) && (dirname[1] == ':'))
-        return true;
-
-    if (fileExists(dirname) == 0) {
-
-        char *bs = strrchr(dirname, '\\');
-        char *fs = strrchr(dirname, '/');
-
-        if ((bs == NULL) && (fs != NULL)) {
-            *fs = 0x00;
-            create_path (dirname);
-            *fs = '/';
-        }
-
-        if ((bs != NULL) && (fs == NULL)) {
-            *bs = 0x00;
-            create_path (dirname);
-            *bs = '\\';
-        }
-
-        if ((bs != NULL) && (fs != NULL)) {
-            if (strlen (bs) > strlen (fs)) {
-                *fs = 0x00; // No slash
-                create_path (dirname);
-                *fs = '/';
-            } else {
-                *bs = 0x00;
-                create_path (dirname);
-                *bs = '\\';
-            }
-
-        }
-
-        if (make_dir(dirname) != 0) {
-           PrintAndLogEx(ERR, "could not create directory.... "_RED_("%s"),dirname);
-           return false;
-        }
-    }
-    return true;
-}
-*/
-
 bool setDefaultPath(savePaths_t pathIndex, const char *path) {
 
     if (pathIndex < spItemCount) {
@@ -262,37 +182,55 @@ static size_t path_size(savePaths_t a) {
 }
 
 char *newfilenamemcopy(const char *preferredName, const char *suffix) {
+    return newfilenamemcopyEx(preferredName, suffix, spDefault);
+}
+
+char *newfilenamemcopyEx(const char *preferredName, const char *suffix, savePaths_t e_save_path) {
     if (preferredName == NULL || suffix == NULL) {
         return NULL;
     }
 
     uint16_t p_namelen = strlen(preferredName);
-    if (str_endswith(preferredName, suffix))
+    if (str_endswith(preferredName, suffix)) {
         p_namelen -= strlen(suffix);
+    }
 
-    // 10: room for filenum to ensure new filename
-    const size_t len = p_namelen + strlen(suffix) + 1 + 10;
+    int save_path_len = path_size(e_save_path);
 
-    int foobar = path_size(spDefault);
-    (void) foobar;
+    // 1: null terminator
+    // 16: room for filenum to ensure new filename
+    // save_path_len + strlen(PATHSEP):  the user preference save paths
+    const size_t len = p_namelen + strlen(suffix) + 1 + 16 + save_path_len + strlen(PATHSEP);
 
     char *fileName = (char *) calloc(len, sizeof(uint8_t));
     if (fileName == NULL) {
         return NULL;
     }
 
+    char *pfn = fileName;
+
+    // user preference save paths
+    if (save_path_len) {
+        snprintf(pfn, save_path_len + strlen(PATHSEP) + 1, "%s%s", g_session.defaultPaths[e_save_path], PATHSEP);
+        pfn += save_path_len + strlen(PATHSEP);
+    }
+
     int num = 1;
 
-    snprintf(fileName, len, "%.*s%s", p_namelen, preferredName, suffix);
+    // modify filename
+    snprintf(pfn, len, "%.*s%s", p_namelen, preferredName, suffix);
+
+    // check complete path/filename if exists
     while (fileExists(fileName)) {
-        snprintf(fileName, len, "%.*s-%d%s", p_namelen, preferredName, num, suffix);
+        // modify filename
+        snprintf(pfn, len, "%.*s-%03d%s", p_namelen, preferredName, num, suffix);
         num++;
     }
 
-    PrintAndLogEx(INFO, "FILE PATH:  %s", fileName);
     return fileName;
 }
 
+// --------- SAVE FILES
 int saveFile(const char *preferredName, const char *suffix, const void *data, size_t datalen) {
 
     if (data == NULL || datalen == 0) {
@@ -304,12 +242,12 @@ int saveFile(const char *preferredName, const char *suffix, const void *data, si
         return PM3_EMALLOC;
     }
 
-    /* We should have a valid filename now, e.g. dumpdata-3.bin */
+    // We should have a valid filename now, e.g. dumpdata-3.bin
 
-    /*Opening file for writing in binary mode*/
+    // Opening file for writing in binary mode
     FILE *f = fopen(fileName, "wb");
     if (!f) {
-        PrintAndLogEx(WARNING, "file not found or locked. '" _YELLOW_("%s")"'", fileName);
+        PrintAndLogEx(WARNING, "file not found or locked `" _YELLOW_("%s") "`", fileName);
         free(fileName);
         return PM3_EFILE;
     }
@@ -321,13 +259,14 @@ int saveFile(const char *preferredName, const char *suffix, const void *data, si
     return PM3_SUCCESS;
 }
 
+// dump file
 int saveFileEML(const char *preferredName, uint8_t *data, size_t datalen, size_t blocksize) {
 
     if (data == NULL || datalen == 0) {
         return PM3_EINVARG;
     }
 
-    char *fileName = newfilenamemcopy(preferredName, ".eml");
+    char *fileName = newfilenamemcopyEx(preferredName, ".eml", spDump);
     if (fileName == NULL) {
         return PM3_EMALLOC;
     }
@@ -336,12 +275,12 @@ int saveFileEML(const char *preferredName, uint8_t *data, size_t datalen, size_t
     int blocks = datalen / blocksize;
     uint16_t currblock = 1;
 
-    /* We should have a valid filename now, e.g. dumpdata-3.bin */
+    // We should have a valid filename now, e.g. dumpdata-3.bin
 
-    /*Opening file for writing in text mode*/
+    // Opening file for writing in text mode
     FILE *f = fopen(fileName, "w+");
     if (!f) {
-        PrintAndLogEx(WARNING, "file not found or locked. '" _YELLOW_("%s")"'", fileName);
+        PrintAndLogEx(WARNING, "file not found or locked `" _YELLOW_("%s") "`", fileName);
         retval = PM3_EFILE;
         goto out;
     }
@@ -371,16 +310,19 @@ out:
     return retval;
 }
 
+// dump file (normally,  we also got preference file, etc)
 int saveFileJSON(const char *preferredName, JSONFileType ftype, uint8_t *data, size_t datalen, void (*callback)(json_t *)) {
-    return saveFileJSONex(preferredName, ftype, data, datalen, true, callback);
+    return saveFileJSONex(preferredName, ftype, data, datalen, true, callback, spDump);
 }
-int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data, size_t datalen, bool verbose, void (*callback)(json_t *)) {
+int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data, size_t datalen, bool verbose, void (*callback)(json_t *), savePaths_t e_save_path) {
 
-    if (data == NULL || datalen == 0) {
-        return PM3_EINVARG;
+    if (ftype != jsfCustom) {
+        if (data == NULL || datalen == 0) {
+            return PM3_EINVARG;
+        }
     }
 
-    char *fileName = newfilenamemcopy(preferredName, ".json");
+    char *fileName = newfilenamemcopyEx(preferredName, ".json", e_save_path);
     if (fileName == NULL) {
         return PM3_EMALLOC;
     }
@@ -602,28 +544,29 @@ int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data,
             break;
         }
         case jsfMfPlusKeys: {
-            JsonSaveStr(root, "FileType", "mfp");
+            JsonSaveStr(root, "FileType", "mfpkeys");
             JsonSaveBufAsHexCompact(root, "$.Card.UID", &data[0], 7);
             JsonSaveBufAsHexCompact(root, "$.Card.SAK", &data[10], 1);
             JsonSaveBufAsHexCompact(root, "$.Card.ATQA", &data[11], 2);
             uint8_t atslen = data[13];
-            if (atslen > 0)
+            if (atslen > 0) {
                 JsonSaveBufAsHexCompact(root, "$.Card.ATS", &data[14], atslen);
+            }
 
-            uint8_t vdata[2][64][16 + 1] = {{{0}}};
-            memcpy(vdata, &data[14 + atslen], 2 * 64 * 17);
+            uint8_t vdata[2][64][17] = {{{0}}};
+            memcpy(vdata, data + (14 + atslen), 2 * 64 * 17);
 
             for (size_t i = 0; i < datalen; i++) {
                 char path[PATH_MAX_LENGTH] = {0};
 
                 if (vdata[0][i][0]) {
-                    snprintf(path, sizeof(path), "$.SectorKeys.%d.KeyA", mfSectorNum(i));
-                    JsonSaveBufAsHexCompact(root, path, &vdata[0][i][1], 16);
+                    snprintf(path, sizeof(path), "$.SectorKeys.%zu.KeyA", i);
+                    JsonSaveBufAsHexCompact(root, path, &vdata[0][i][1], AES_KEY_LEN);
                 }
 
                 if (vdata[1][i][0]) {
-                    snprintf(path, sizeof(path), "$.SectorKeys.%d.KeyB", mfSectorNum(i));
-                    JsonSaveBufAsHexCompact(root, path, &vdata[1][i][1], 16);
+                    snprintf(path, sizeof(path), "$.SectorKeys.%zu.KeyB", i);
+                    JsonSaveBufAsHexCompact(root, path, &vdata[1][i][1], AES_KEY_LEN);
                 }
             }
             break;
@@ -719,7 +662,7 @@ int saveFileJSONrootEx(const char *preferredName, void *root, size_t flags, bool
     if (overwrite)
         filename = filenamemcopy(preferredName, ".json");
     else
-        filename = newfilenamemcopy(preferredName, ".json");
+        filename = newfilenamemcopyEx(preferredName, ".json", spDump);
 
     if (filename == NULL)
         return PM3_EMALLOC;
@@ -739,13 +682,14 @@ int saveFileJSONrootEx(const char *preferredName, void *root, size_t flags, bool
     return PM3_EFILE;
 }
 
+// wave file of trace,
 int saveFileWAVE(const char *preferredName, const int *data, size_t datalen) {
 
     if (data == NULL || datalen == 0) {
         return PM3_EINVARG;
     }
 
-    char *fileName = newfilenamemcopy(preferredName, ".wav");
+    char *fileName = newfilenamemcopyEx(preferredName, ".wav", spTrace);
     if (fileName == NULL) {
         return PM3_EMALLOC;
     }
@@ -770,7 +714,7 @@ int saveFileWAVE(const char *preferredName, const int *data, size_t datalen) {
 
     FILE *wave_file = fopen(fileName, "wb");
     if (!wave_file) {
-        PrintAndLogEx(WARNING, "file not found or locked. "_YELLOW_("'%s'"), fileName);
+        PrintAndLogEx(WARNING, "file not found or locked `" _YELLOW_("%s") "`", fileName);
         retval = PM3_EFILE;
         goto out;
     }
@@ -791,13 +735,14 @@ out:
     return retval;
 }
 
+// Signal trace file, PM3
 int saveFilePM3(const char *preferredName, int *data, size_t datalen) {
 
     if (data == NULL || datalen == 0) {
         return PM3_EINVARG;
     }
 
-    char *fileName = newfilenamemcopy(preferredName, ".pm3");
+    char *fileName = newfilenamemcopyEx(preferredName, ".pm3", spTrace);
     if (fileName == NULL) {
         return PM3_EMALLOC;
     }
@@ -806,7 +751,7 @@ int saveFilePM3(const char *preferredName, int *data, size_t datalen) {
 
     FILE *f = fopen(fileName, "w");
     if (!f) {
-        PrintAndLogEx(WARNING, "file not found or locked. "_YELLOW_("'%s'"), fileName);
+        PrintAndLogEx(WARNING, "file not found or locked `" _YELLOW_("%s") "`", fileName);
         retval = PM3_EFILE;
         goto out;
     }
@@ -824,11 +769,12 @@ out:
     return retval;
 }
 
+// key file dump
 int createMfcKeyDump(const char *preferredName, uint8_t sectorsCnt, sector_t *e_sector) {
 
     if (e_sector == NULL) return PM3_EINVARG;
 
-    char *fileName = newfilenamemcopy(preferredName, ".bin");
+    char *fileName = newfilenamemcopyEx(preferredName, ".bin", spDump);
     if (fileName == NULL) return PM3_EMALLOC;
 
     FILE *f = fopen(fileName, "wb");
@@ -861,71 +807,12 @@ int createMfcKeyDump(const char *preferredName, uint8_t sectorsCnt, sector_t *e_
     fflush(f);
     fclose(f);
     PrintAndLogEx(SUCCESS, "Found keys have been dumped to " _YELLOW_("%s"), fileName);
-    PrintAndLogEx(INFO, "FYI! --> " _YELLOW_("0xFFFFFFFFFFFF") " <-- has been inserted for unknown keys where " _YELLOW_("res") " is " _YELLOW_("0"));
+    PrintAndLogEx(INFO, "FYI! --> " _YELLOW_("0xFFFFFFFFFFFF") " <-- has been inserted for unknown keys where " _YELLOW_("res") " is " _RED_("0"));
     free(fileName);
     return PM3_SUCCESS;
 }
 
-int loadFile(const char *preferredName, const char *suffix, void *data, size_t maxdatalen, size_t *datalen) {
-
-    if (data == NULL) return 1;
-    char *fileName = filenamemcopy(preferredName, suffix);
-    if (fileName == NULL) return PM3_EINVARG;
-
-    int retval = PM3_SUCCESS;
-
-    FILE *f = fopen(fileName, "rb");
-    if (!f) {
-        PrintAndLogEx(WARNING, "file not found or locked. '" _YELLOW_("%s")"'", fileName);
-        free(fileName);
-        return PM3_EFILE;
-    }
-
-    // get filesize in order to malloc memory
-    fseek(f, 0, SEEK_END);
-    long fsize = ftell(f);
-    fseek(f, 0, SEEK_SET);
-
-    if (fsize <= 0) {
-        PrintAndLogEx(FAILED, "error, when getting filesize");
-        retval = PM3_EFILE;
-        goto out;
-    }
-
-    uint8_t *dump = calloc(fsize, sizeof(uint8_t));
-    if (!dump) {
-        PrintAndLogEx(FAILED, "error, cannot allocate memory");
-        retval = PM3_EMALLOC;
-        goto out;
-    }
-
-    size_t bytes_read = fread(dump, 1, fsize, f);
-
-    if (bytes_read != fsize) {
-        PrintAndLogEx(FAILED, "error, bytes read mismatch file size");
-        free(dump);
-        retval = PM3_EFILE;
-        goto out;
-    }
-
-    if (bytes_read > maxdatalen) {
-        PrintAndLogEx(WARNING, "Warning, bytes read exceed calling array limit. Max bytes is %zu bytes", maxdatalen);
-        bytes_read = maxdatalen;
-    }
-
-    memcpy((data), dump, bytes_read);
-    free(dump);
-
-    PrintAndLogEx(SUCCESS, "loaded " _YELLOW_("%zu") " bytes from binary file " _YELLOW_("%s"), bytes_read, fileName);
-
-    *datalen = bytes_read;
-
-out:
-    fclose(f);
-    free(fileName);
-    return retval;
-}
-
+// --------- LOAD FILES
 int loadFile_safe(const char *preferredName, const char *suffix, void **pdata, size_t *datalen) {
     return loadFile_safeEx(preferredName, suffix, pdata, datalen, true);
 }
@@ -939,7 +826,7 @@ int loadFile_safeEx(const char *preferredName, const char *suffix, void **pdata,
 
     FILE *f = fopen(path, "rb");
     if (!f) {
-        PrintAndLogEx(WARNING, "file not found or locked. '" _YELLOW_("%s")"'", path);
+        PrintAndLogEx(WARNING, "file not found or locked `" _YELLOW_("%s") "`", path);
         free(path);
         return PM3_EFILE;
     }
@@ -975,72 +862,12 @@ int loadFile_safeEx(const char *preferredName, const char *suffix, void **pdata,
 
     *datalen = bytes_read;
 
-    if (verbose)
+    if (verbose) {
         PrintAndLogEx(SUCCESS, "loaded " _YELLOW_("%zu") " bytes from binary file " _YELLOW_("%s"), bytes_read, preferredName);
+    }
     return PM3_SUCCESS;
 }
 
-int loadFileEML(const char *preferredName, void *data, size_t *datalen) {
-
-    if (data == NULL) return PM3_EINVARG;
-
-    char *fileName = filenamemcopy(preferredName, ".eml");
-    if (fileName == NULL) return PM3_EMALLOC;
-
-    size_t counter = 0;
-    int retval = PM3_SUCCESS, hexlen = 0;
-
-    FILE *f = fopen(fileName, "r");
-    if (!f) {
-        PrintAndLogEx(WARNING, "file not found or locked. '" _YELLOW_("%s")"'", fileName);
-        retval = PM3_EFILE;
-        goto out;
-    }
-
-    // 128 + 2 newline chars + 1 null terminator
-    char line[131];
-    memset(line, 0, sizeof(line));
-    uint8_t buf[64] = {0x00};
-
-    uint8_t *udata = (uint8_t *)data;
-
-    while (!feof(f)) {
-
-        memset(line, 0, sizeof(line));
-
-        if (fgets(line, sizeof(line), f) == NULL) {
-            if (feof(f))
-                break;
-
-            fclose(f);
-            PrintAndLogEx(FAILED, "File reading error.");
-            retval = PM3_EFILE;
-            goto out;
-        }
-
-        if (line[0] == '#')
-            continue;
-
-        strcleanrn(line, sizeof(line));
-
-        int res = param_gethex_to_eol(line, 0, buf, sizeof(buf), &hexlen);
-        if (res == 0) {
-            memcpy(udata + counter, buf, hexlen);
-            counter += hexlen;
-        } else {
-            retval = PM3_ESOFT;
-        }
-    }
-    fclose(f);
-    PrintAndLogEx(SUCCESS, "loaded " _YELLOW_("%zu") " bytes from text file " _YELLOW_("%s"), counter, fileName);
-
-    if (datalen)
-        *datalen = counter;
-
-out:
-    free(fileName);
-    return retval;
-}
 int loadFileEML_safe(const char *preferredName, void **pdata, size_t *datalen) {
     char *path;
     int res = searchFile(&path, RESOURCES_SUBDIR, preferredName, "", false);
@@ -1050,7 +877,7 @@ int loadFileEML_safe(const char *preferredName, void **pdata, size_t *datalen) {
 
     FILE *f = fopen(path, "r");
     if (!f) {
-        PrintAndLogEx(WARNING, "file not found or locked. '" _YELLOW_("%s")"'", path);
+        PrintAndLogEx(WARNING, "file not found or locked `" _YELLOW_("%s") "`", path);
         free(path);
         return PM3_EFILE;
     }
@@ -1127,6 +954,93 @@ int loadFileEML_safe(const char *preferredName, void **pdata, size_t *datalen) {
     return retval;
 }
 
+int loadFileMCT_safe(const char *preferredName, void **pdata, size_t *datalen) {
+    char *path;
+    int res = searchFile(&path, RESOURCES_SUBDIR, preferredName, "", false);
+    if (res != PM3_SUCCESS) {
+        return PM3_EFILE;
+    }
+
+    FILE *f = fopen(path, "r");
+    if (!f) {
+        PrintAndLogEx(WARNING, "file not found or locked `" _YELLOW_("%s") "`", path);
+        free(path);
+        return PM3_EFILE;
+    }
+    free(path);
+
+    // get filesize in order to malloc memory
+    fseek(f, 0, SEEK_END);
+    long fsize = ftell(f);
+    fseek(f, 0, SEEK_SET);
+
+    if (fsize <= 0) {
+        PrintAndLogEx(FAILED, "error, when getting filesize");
+        fclose(f);
+        return PM3_EFILE;
+    }
+
+    *pdata = calloc(fsize, sizeof(uint8_t));
+    if (!*pdata) {
+        PrintAndLogEx(FAILED, "error, cannot allocate memory");
+        fclose(f);
+        return PM3_EMALLOC;
+    }
+
+    // 128 + 2 newline chars + 1 null terminator
+    char line[131];
+    memset(line, 0, sizeof(line));
+    uint8_t buf[64] = {0x00};
+    size_t counter = 0;
+    int retval = PM3_SUCCESS, hexlen = 0;
+
+    uint8_t *tmp = (uint8_t *)*pdata;
+
+    while (!feof(f)) {
+
+        memset(line, 0, sizeof(line));
+
+        if (fgets(line, sizeof(line), f) == NULL) {
+            if (feof(f))
+                break;
+
+            fclose(f);
+            PrintAndLogEx(FAILED, "File reading error.");
+            return PM3_EFILE;
+        }
+
+        // skip lines like "+Sector:"
+        if (line[0] == '+')
+            continue;
+
+        strcleanrn(line, sizeof(line));
+
+        res = param_gethex_to_eol(line, 0, buf, sizeof(buf), &hexlen);
+        if (res == 0) {
+            memcpy(tmp + counter, buf, hexlen);
+            counter += hexlen;
+        } else {
+            retval = PM3_ESOFT;
+        }
+    }
+    fclose(f);
+    PrintAndLogEx(SUCCESS, "loaded " _YELLOW_("%zu") " bytes from MCT file " _YELLOW_("%s"), counter, preferredName);
+
+
+    uint8_t *newdump = realloc(*pdata, counter);
+    if (newdump == NULL) {
+        free(*pdata);
+        return PM3_EMALLOC;
+    } else {
+        *pdata = newdump;
+    }
+
+    if (datalen)
+        *datalen = counter;
+
+    return retval;
+}
+
 int loadFileJSON(const char *preferredName, void *data, size_t maxdatalen, size_t *datalen, void (*callback)(json_t *)) {
     return loadFileJSONex(preferredName, data, maxdatalen, datalen, true, callback);
 }
@@ -1384,6 +1298,40 @@ int loadFileJSONex(const char *preferredName, void *data, size_t maxdatalen, siz
 
         *datalen += sptr;
     }
+
+    if (!strcmp(ctype, "mfpkeys")) {
+
+        JsonLoadBufAsHex(root, "$.Card.UID", udata.bytes, 7, datalen);
+        JsonLoadBufAsHex(root, "$.Card.SAK", udata.bytes + 10, 1, datalen);
+        JsonLoadBufAsHex(root, "$.Card.ATQA", udata.bytes + 11, 2, datalen);
+        uint8_t atslen = udata.bytes[13];
+        if (atslen > 0) {
+            JsonLoadBufAsHex(root, "$.Card.ATS", udata.bytes + 14, atslen, datalen);
+        }
+
+        size_t sptr = (14 + atslen);
+
+        // memcpy(vdata, udata.bytes + (14 + atslen), 2 * 64 * 17);
+        for (size_t i = 0; i < 64; i++) {
+
+            if ((sptr + (AES_KEY_LEN * 2)) > maxdatalen) {
+                break;
+            }
+
+            size_t offset = (14 + atslen) + (i * 2 * AES_KEY_LEN);
+
+            char blocks[40] = {0};
+            snprintf(blocks, sizeof(blocks), "$.SectorKeys.%zu.KeyA", i);
+            JsonLoadBufAsHex(root, blocks, udata.bytes + offset, AES_KEY_LEN, datalen);
+
+            snprintf(blocks, sizeof(blocks), "$.SectorKeys.%zu.KeyB", i);
+            JsonLoadBufAsHex(root, blocks, udata.bytes + offset + AES_KEY_LEN, AES_KEY_LEN, datalen);
+
+            sptr += (2 * AES_KEY_LEN);
+        }
+        *datalen += sptr;
+    }
+
 out:
 
     if (callback != NULL) {
@@ -1427,6 +1375,7 @@ int loadFileJSONroot(const char *preferredName, void **proot, bool verbose) {
     return retval;
 }
 
+// iceman:  todo - move all unsafe functions like this from client source.
 int loadFileDICTIONARY(const char *preferredName, void *data, size_t *datalen, uint8_t keylen, uint32_t *keycnt) {
     // t5577 == 4 bytes
     // mifare == 6 bytes
@@ -1463,7 +1412,7 @@ int loadFileDICTIONARYEx(const char *preferredName, void *data, size_t maxdatale
 
     FILE *f = fopen(path, "r");
     if (!f) {
-        PrintAndLogEx(WARNING, "file not found or locked. '" _YELLOW_("%s")"'", path);
+        PrintAndLogEx(WARNING, "file not found or locked `" _YELLOW_("%s") "`", path);
         retval = PM3_EFILE;
         goto out;
     }
@@ -1566,7 +1515,7 @@ int loadFileDICTIONARY_safe(const char *preferredName, void **pdata, uint8_t key
 
     FILE *f = fopen(path, "r");
     if (!f) {
-        PrintAndLogEx(WARNING, "file not found or locked. '" _YELLOW_("%s")"'", path);
+        PrintAndLogEx(WARNING, "file not found or locked `" _YELLOW_("%s") "`", path);
         retval = PM3_EFILE;
         goto out;
     }
@@ -1619,6 +1568,57 @@ out:
     return retval;
 }
 
+int loadFileBinaryKey(const char *preferredName, const char *suffix, void **keya, void **keyb, size_t *alen, size_t *blen) {
+
+    char *path;
+    int res = searchFile(&path, RESOURCES_SUBDIR, preferredName, suffix, false);
+    if (res != PM3_SUCCESS) {
+        return PM3_EFILE;
+    }
+
+    FILE *f = fopen(path, "rb");
+    if (!f) {
+        PrintAndLogEx(WARNING, "file not found or locked `" _YELLOW_("%s") "`", path);
+        free(path);
+        return PM3_EFILE;
+    }
+    free(path);
+
+    // get filesize in order to malloc memory
+    fseek(f, 0, SEEK_END);
+    long fsize = ftell(f);
+    fseek(f, 0, SEEK_SET);
+
+    if (fsize <= 0) {
+        PrintAndLogEx(FAILED, "error, when getting filesize");
+        fclose(f);
+        return PM3_EFILE;
+    }
+
+    // Half is KEY A,  half is KEY B
+    fsize /= 2;
+
+    *keya = calloc(fsize, sizeof(uint8_t));
+    if (*keya == NULL) {
+        PrintAndLogEx(FAILED, "error, cannot allocate memory");
+        fclose(f);
+        return PM3_EMALLOC;
+    }
+
+    *alen = fread(*keya, 1, fsize, f);
+
+    *keyb = calloc(fsize, sizeof(uint8_t));
+    if (*keyb == NULL) {
+        PrintAndLogEx(FAILED, "error, cannot allocate memory");
+        fclose(f);
+        return PM3_EMALLOC;
+    }
+
+    *blen = fread(*keyb, 1, fsize, f);
+    fclose(f);
+    return PM3_SUCCESS;
+}
+
 mfu_df_e detect_mfu_dump_format(uint8_t **dump, size_t *dumplen, bool verbose) {
 
     mfu_df_e retval = MFU_DF_UNKNOWN;
@@ -1851,51 +1851,84 @@ int searchAndList(const char *pm3dir, const char *ext) {
 }
 
 static int searchFinalFile(char **foundpath, const char *pm3dir, const char *searchname, bool silent) {
-    if ((foundpath == NULL) || (pm3dir == NULL) || (searchname == NULL)) return PM3_ESOFT;
+
+    if ((foundpath == NULL) || (pm3dir == NULL) || (searchname == NULL)) {
+        return PM3_ESOFT;
+    }
+
     // explicit absolute (/) or relative path (./) => try only to match it directly
     char *filename = calloc(strlen(searchname) + 1, sizeof(char));
-    if (filename == NULL) return PM3_EMALLOC;
+    if (filename == NULL) {
+        return PM3_EMALLOC;
+    }
+
     strcpy(filename, searchname);
     if ((g_debugMode == 2) && (!silent)) {
-        PrintAndLogEx(INFO, "Searching %s", filename);
+        PrintAndLogEx(INFO, "pm3dir...... %s", pm3dir);
+        PrintAndLogEx(INFO, "Searching... %s", filename);
     }
+
+    // try implicit relative path
+    PrintAndLogEx(DEBUG, "Searching implicit relative paths");
+    if (fileExists(filename)) {
+        *foundpath = filename;
+        if ((g_debugMode == 2) && (!silent)) {
+            PrintAndLogEx(INFO, "Found %s", *foundpath);
+        }
+        return PM3_SUCCESS;
+    }
+
     if (((strlen(filename) > 1) && (filename[0] == '/')) ||
             ((strlen(filename) > 2) && (filename[0] == '.') && (filename[1] == '/'))) {
-        if (fileExists(filename)) {
-            *foundpath = filename;
+        goto out;
+    }
+
+    // try the session paths
+    PrintAndLogEx(DEBUG, "Searching preferences paths");
+    for (int i = 0; i < spItemCount; i++) {
+
+        size_t sn = strlen(g_session.defaultPaths[i]) + strlen(filename) + strlen(PATHSEP) + 1;
+        char *default_path = calloc(sn, sizeof(char));
+        if (default_path == NULL) {
+            goto out;
+        }
+
+        snprintf(default_path, sn, "%s%s%s", g_session.defaultPaths[i], PATHSEP, filename);
+
+        if ((g_debugMode == 2) && (!silent)) {
+            PrintAndLogEx(INFO, "Searching %s", default_path);
+        }
+
+        if (fileExists(default_path)) {
+            free(filename);
+            *foundpath = default_path;
             if ((g_debugMode == 2) && (!silent)) {
                 PrintAndLogEx(INFO, "Found %s", *foundpath);
             }
             return PM3_SUCCESS;
         } else {
-            goto out;
+            free(default_path);
         }
     }
-    // else
 
-    // try implicit relative path
-    {
-        if (fileExists(filename)) {
-            *foundpath = filename;
-            if ((g_debugMode == 2) && (!silent)) {
-                PrintAndLogEx(INFO, "Found %s", *foundpath);
-            }
-            return PM3_SUCCESS;
-        }
-    }
     // try pm3 dirs in user .proxmark3 (user mode)
+    PrintAndLogEx(DEBUG, "Searching user .proxmark3 paths");
     const char *user_path = get_my_user_directory();
     if (user_path != NULL) {
         char *path = calloc(strlen(user_path) + strlen(PM3_USER_DIRECTORY) + strlen(pm3dir) + strlen(filename) + 1, sizeof(char));
-        if (path == NULL)
+        if (path == NULL) {
             goto out;
+        }
+
         strcpy(path, user_path);
         strcat(path, PM3_USER_DIRECTORY);
         strcat(path, pm3dir);
         strcat(path, filename);
+
         if ((g_debugMode == 2) && (!silent)) {
             PrintAndLogEx(INFO, "Searching %s", path);
         }
+
         if (fileExists(path)) {
             free(filename);
             *foundpath = path;
@@ -1907,7 +1940,9 @@ static int searchFinalFile(char **foundpath, const char *pm3dir, const char *sea
             free(path);
         }
     }
+
     // try pm3 dirs in current client workdir (dev mode)
+    PrintAndLogEx(DEBUG, "Searching current workdir paths");
     const char *exec_path = get_my_executable_directory();
     if ((exec_path != NULL) &&
             ((strcmp(DICTIONARIES_SUBDIR, pm3dir) == 0) ||
@@ -1940,23 +1975,28 @@ static int searchFinalFile(char **foundpath, const char *pm3dir, const char *sea
             free(path);
         }
     }
+
     // try pm3 dirs in current repo workdir (dev mode)
+    PrintAndLogEx(DEBUG, "Searching PM3 dirs in current workdir");
     if ((exec_path != NULL) &&
             ((strcmp(TRACES_SUBDIR, pm3dir) == 0) ||
              (strcmp(FIRMWARES_SUBDIR, pm3dir) == 0) ||
              (strcmp(BOOTROM_SUBDIR, pm3dir) == 0) ||
              (strcmp(FULLIMAGE_SUBDIR, pm3dir) == 0))) {
-        const char *above = "../";
-        char *path = calloc(strlen(exec_path) + strlen(above) + strlen(pm3dir) + strlen(filename) + 1, sizeof(char));
-        if (path == NULL)
+        char *path = calloc(strlen(exec_path) + strlen(ABOVE) + strlen(pm3dir) + strlen(filename) + 1, sizeof(char));
+        if (path == NULL) {
             goto out;
+        }
+
         strcpy(path, exec_path);
-        strcat(path, above);
+        strcat(path, ABOVE);
         strcat(path, pm3dir);
         strcat(path, filename);
+
         if ((g_debugMode == 2) && (!silent)) {
             PrintAndLogEx(INFO, "Searching %s", path);
         }
+
         if (fileExists(path)) {
             free(filename);
             *foundpath = path;
@@ -1968,18 +2008,24 @@ static int searchFinalFile(char **foundpath, const char *pm3dir, const char *sea
             free(path);
         }
     }
+
     // try pm3 dirs in pm3 installation dir (install mode)
+    PrintAndLogEx(DEBUG, "Searching PM3 installation dir paths");
     if (exec_path != NULL) {
         char *path = calloc(strlen(exec_path) + strlen(PM3_SHARE_RELPATH) + strlen(pm3dir) + strlen(filename) + 1, sizeof(char));
-        if (path == NULL)
+        if (path == NULL) {
             goto out;
+        }
+
         strcpy(path, exec_path);
         strcat(path, PM3_SHARE_RELPATH);
         strcat(path, pm3dir);
         strcat(path, filename);
+
         if ((g_debugMode == 2) && (!silent)) {
             PrintAndLogEx(INFO, "Searching %s", path);
         }
+
         if (fileExists(path)) {
             free(filename);
             *foundpath = path;
@@ -2015,15 +2061,15 @@ int searchFile(char **foundpath, const char *pm3dir, const char *searchname, con
         free(filename);
         return PM3_EFILE;
     }
+
     int res = searchFinalFile(foundpath, pm3dir, filename, silent);
     if (res != PM3_SUCCESS) {
-        if ((res == PM3_EFILE) && (!silent))
+        if ((res == PM3_EFILE) && (!silent)) {
             PrintAndLogEx(FAILED, "Error - can't find `" _YELLOW_("%s") "`", filename);
-        free(filename);
-        return res;
+        }
     }
     free(filename);
-    return PM3_SUCCESS;
+    return res;
 }
 
 int pm3_load_dump(const char *fn, void **pdump, size_t *dumplen, size_t maxdumplen) {
@@ -2055,10 +2101,14 @@ int pm3_load_dump(const char *fn, void **pdump, size_t *dumplen, size_t maxdumpl
             PrintAndLogEx(ERR, "Error: Only BIN/EML/JSON formats allowed");
             return PM3_EINVARG;
         }
+        case MCT: {
+            res = loadFileMCT_safe(fn, pdump, dumplen);
+            break;
+        }
     }
 
     if (res != PM3_SUCCESS) {
-        PrintAndLogEx(FAILED, "File: " _YELLOW_("%s") ": not found or locked.", fn);
+        PrintAndLogEx(WARNING, "file not found or locked `" _YELLOW_("%s") "`", fn);
         return PM3_EFILE;
     }
 
@@ -2068,7 +2118,7 @@ int pm3_load_dump(const char *fn, void **pdump, size_t *dumplen, size_t maxdumpl
 int pm3_save_dump(const char *fn, uint8_t *d, size_t n, JSONFileType jsft, size_t blocksize) {
 
     if (d == NULL || n == 0) {
-        PrintAndLogEx(INFO, "No data to save. Skipping...");
+        PrintAndLogEx(INFO, "No data to save, skipping...");
         return PM3_EINVARG;
     }
 
