@@ -56,15 +56,16 @@ typedef enum {
     EML,
     JSON,
     DICTIONARY,
+    MCT,
 } DumpFileType_t;
 
 int fileExists(const char *filename);
-//bool create_path(const char *dirname);
 
 // set a path in the path list g_session.defaultPaths
 bool setDefaultPath(savePaths_t pathIndex, const char *path);
 
 char *newfilenamemcopy(const char *preferredName, const char *suffix);
+char *newfilenamemcopyEx(const char *preferredName, const char *suffix, savePaths_t save_path);
 
 /**
  * @brief Utility function to save data to a binary file. This method takes a preferred name, but if that
@@ -104,7 +105,7 @@ int saveFileEML(const char *preferredName, uint8_t *data, size_t datalen, size_t
  * @return 0 for ok, 1 for failz
  */
 int saveFileJSON(const char *preferredName, JSONFileType ftype, uint8_t *data, size_t datalen, void (*callback)(json_t *));
-int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data, size_t datalen, bool verbose, void (*callback)(json_t *));
+int saveFileJSONex(const char *preferredName, JSONFileType ftype, uint8_t *data, size_t datalen, bool verbose, void (*callback)(json_t *), savePaths_t e_save_path);
 int saveFileJSONroot(const char *preferredName, void *root, size_t flags, bool verbose);
 int saveFileJSONrootEx(const char *preferredName, void *root, size_t flags, bool verbose, bool overwrite);
 /** STUB
@@ -141,20 +142,6 @@ int saveFilePM3(const char *preferredName, int *data, size_t datalen);
  */
 int createMfcKeyDump(const char *preferredName, uint8_t sectorsCnt, sector_t *e_sector);
 
-/**
- * @brief Utility function to load data from a binary file. This method takes a preferred name.
- * E.g. dumpdata-15.bin
- *
- * @param preferredName
- * @param suffix the file suffix. Including the ".".
- * @param data The data array to store the loaded bytes from file
- * @param maxdatalen the number of bytes that your data array has
- * @param datalen the number of bytes loaded from file
- * @return PM3_SUCCESS for ok, PM3_E* for failz
-*/
-int loadFile(const char *preferredName, const char *suffix, void *data, size_t maxdatalen, size_t *datalen);
-
-
 /**
  * @brief Utility function to load data from a binary file. This method takes a preferred name.
  * E.g. dumpdata-15.bin,  tries to search for it,  and allocated memory.
@@ -176,9 +163,19 @@ int loadFile_safeEx(const char *preferredName, const char *suffix, void **pdata,
  * @param datalen the number of bytes loaded from file
  * @return 0 for ok, 1 for failz
 */
-int loadFileEML(const char *preferredName, void *data, size_t *datalen);
 int loadFileEML_safe(const char *preferredName, void **pdata, size_t *datalen);
 
+/**
+ * @brief  Utility function to load data from a textfile (MCT). This method takes a preferred name.
+ * E.g. dumpdata-15.mct
+ *
+ * @param preferredName
+ * @param data The data array to store the loaded bytes from file
+ * @param datalen the number of bytes loaded from file
+ * @return 0 for ok, 1 for failz
+*/
+int loadFileMCT_safe(const char *preferredName, void **pdata, size_t *datalen);
+
 /**
  * @brief  Utility function to load data from a JSON textfile. This method takes a preferred name.
  * E.g. dumpdata-15.json
@@ -236,6 +233,7 @@ int loadFileDICTIONARYEx(const char *preferredName, void *data, size_t maxdatale
 */
 int loadFileDICTIONARY_safe(const char *preferredName, void **pdata, uint8_t keylen, uint32_t *keycnt);
 
+int loadFileBinaryKey(const char *preferredName, const char *suffix, void **keya, void **keyb, size_t *alen, size_t *blen);
 
 typedef enum {
     MFU_DF_UNKNOWN,
@@ -276,5 +274,20 @@ DumpFileType_t getfiletype(const char *filename);
  */
 int pm3_load_dump(const char *fn, void **pdump, size_t *dumplen, size_t maxdumplen);
 
+
+/** STUB
+ * @brief Utility function to save data to three file files (BIN/EML/JSON).
+ * It also tries to save according to user preferences set dump folder paths.
+ * E.g. dumpdata.bin
+ * E.g. dumpdata.eml
+ * E.g. dumpdata.json
+
+ * @param fn
+ * @param d The binary data to write to the file
+ * @param n the length of the data
+ * @param jsft json format type for the different memory cards (MFC, MFUL, LEGIC, 14B, 15, ICLASS etc)
+ * @param blocksize
+ * @return PM3_SUCCESS if OK
+ */
 int pm3_save_dump(const char *fn, uint8_t *d, size_t n, JSONFileType jsft, size_t blocksize);
 #endif // FILEUTILS_H
