@@ -24,9 +24,9 @@
 #include "ui.h"
 #include "crypto/libpcrypto.h"
 
-static bool VerboseMode = false;
+static bool g_verbose_mode = false;
 void mfpSetVerboseMode(bool verbose) {
-    VerboseMode = verbose;
+    g_verbose_mode = verbose;
 }
 
 static const PlusErrorsElm_t PlusErrors[] = {
@@ -289,8 +289,9 @@ int MifareAuth4(mf4Session_t *mf4session, uint8_t *keyn, uint8_t *key, bool acti
     uint8_t kenc[16] = {0};
     memcpy(&kenc[0], &RndA[11], 5);
     memcpy(&kenc[5], &RndB[11], 5);
-    for (int i = 0; i < 5; i++)
+    for (int i = 0; i < 5; i++) {
         kenc[10 + i] = RndA[4 + i] ^ RndB[4 + i];
+    }
     kenc[15] = 0x11;
 
     aes_encode(NULL, key, kenc, kenc, 16);
@@ -301,8 +302,9 @@ int MifareAuth4(mf4Session_t *mf4session, uint8_t *keyn, uint8_t *key, bool acti
     uint8_t kmac[16] = {0};
     memcpy(&kmac[0], &RndA[7], 5);
     memcpy(&kmac[5], &RndB[7], 5);
-    for (int i = 0; i < 5; i++)
+    for (int i = 0; i < 5; i++) {
         kmac[10 + i] = RndA[0 + i] ^ RndB[0 + i];
+    }
     kmac[15] = 0x22;
 
     aes_encode(NULL, key, kmac, kmac, 16);
@@ -338,12 +340,12 @@ int MifareAuth4(mf4Session_t *mf4session, uint8_t *keyn, uint8_t *key, bool acti
 }
 
 static int intExchangeRAW14aPlus(uint8_t *datain, int datainlen, bool activateField, bool leaveSignalON, uint8_t *dataout, int maxdataoutlen, int *dataoutlen) {
-    if (VerboseMode)
+    if (g_verbose_mode)
         PrintAndLogEx(INFO, ">>> %s", sprint_hex(datain, datainlen));
 
     int res = ExchangeRAW14a(datain, datainlen, activateField, leaveSignalON, dataout, maxdataoutlen, dataoutlen, false);
 
-    if (VerboseMode)
+    if (g_verbose_mode)
         PrintAndLogEx(INFO, "<<< %s", sprint_hex(dataout, *dataoutlen));
 
     return res;
@@ -365,7 +367,7 @@ int MFPCommitPerso(bool activateField, bool leaveSignalON, uint8_t *dataout, int
 int MFPReadBlock(mf4Session_t *mf4session, bool plain, uint8_t blockNum, uint8_t blockCount, bool activateField, bool leaveSignalON, uint8_t *dataout, int maxdataoutlen, int *dataoutlen, uint8_t *mac) {
     uint8_t rcmd[4 + 8] = {(plain ? (0x37) : (0x33)), blockNum, 0x00, blockCount};
     if (!plain && mf4session)
-        CalculateMAC(mf4session, mtypReadCmd, blockNum, blockCount, rcmd, 4, &rcmd[4], VerboseMode);
+        CalculateMAC(mf4session, mtypReadCmd, blockNum, blockCount, rcmd, 4, &rcmd[4], g_verbose_mode);
 
     int res = intExchangeRAW14aPlus(rcmd, plain ? 4 : sizeof(rcmd), activateField, leaveSignalON, dataout, maxdataoutlen, dataoutlen);
     if (res)
@@ -375,7 +377,7 @@ int MFPReadBlock(mf4Session_t *mf4session, bool plain, uint8_t blockNum, uint8_t
         mf4session->R_Ctr++;
 
     if (mf4session && mac && *dataoutlen > 11)
-        CalculateMAC(mf4session, mtypReadResp, blockNum, blockCount, dataout, *dataoutlen - 8 - 2, mac, VerboseMode);
+        CalculateMAC(mf4session, mtypReadResp, blockNum, blockCount, dataout, *dataoutlen - 8 - 2, mac, g_verbose_mode);
 
     return 0;
 }
@@ -384,7 +386,7 @@ int MFPWriteBlock(mf4Session_t *mf4session, uint8_t blockNum, uint8_t *data, boo
     uint8_t rcmd[1 + 2 + 16 + 8] = {0xA3, blockNum, 0x00};
     memmove(&rcmd[3], data, 16);
     if (mf4session)
-        CalculateMAC(mf4session, mtypWriteCmd, blockNum, 1, rcmd, 19, &rcmd[19], VerboseMode);
+        CalculateMAC(mf4session, mtypWriteCmd, blockNum, 1, rcmd, 19, &rcmd[19], g_verbose_mode);
 
     int res = intExchangeRAW14aPlus(rcmd, sizeof(rcmd), activateField, leaveSignalON, dataout, maxdataoutlen, dataoutlen);
     if (res)
@@ -394,7 +396,7 @@ int MFPWriteBlock(mf4Session_t *mf4session, uint8_t blockNum, uint8_t *data, boo
         mf4session->W_Ctr++;
 
     if (mf4session && mac && *dataoutlen > 3)
-        CalculateMAC(mf4session, mtypWriteResp, blockNum, 1, dataout, *dataoutlen, mac, VerboseMode);
+        CalculateMAC(mf4session, mtypWriteResp, blockNum, 1, dataout, *dataoutlen, mac, g_verbose_mode);
 
     return 0;
 }
@@ -407,12 +409,12 @@ int mfpReadSector(uint8_t sectorNo, uint8_t keyType, uint8_t *key, uint8_t *data
     keyn[0] = uKeyNum >> 8;
     keyn[1] = uKeyNum & 0xff;
     if (verbose)
-        PrintAndLogEx(INFO, "--sector[%d]:%02x key:%04x", mfNumBlocksPerSector(sectorNo), sectorNo, uKeyNum);
+        PrintAndLogEx(INFO, "--sector[%u]:%02x key:%04x", mfNumBlocksPerSector(sectorNo), sectorNo, uKeyNum);
 
     mf4Session_t _session;
     int res = MifareAuth4(&_session, keyn, key, true, true, true, verbose, false);
     if (res) {
-        PrintAndLogEx(ERR, "Sector %d authentication error: %d", sectorNo, res);
+        PrintAndLogEx(ERR, "Sector %u authentication error: %d", sectorNo, res);
         return res;
     }
 
@@ -423,18 +425,18 @@ int mfpReadSector(uint8_t sectorNo, uint8_t keyType, uint8_t *key, uint8_t *data
     for (int n = firstBlockNo; n < firstBlockNo + mfNumBlocksPerSector(sectorNo); n++) {
         res = MFPReadBlock(&_session, plain, n & 0xff, 1, false, true, data, sizeof(data), &datalen, mac);
         if (res) {
-            PrintAndLogEx(ERR, "Sector %d read error: %d", sectorNo, res);
+            PrintAndLogEx(ERR, "Sector %u read error: %d", sectorNo, res);
             DropField();
             return res;
         }
 
         if (datalen && data[0] != 0x90) {
-            PrintAndLogEx(ERR, "Sector %d card read error: %02x %s", sectorNo, data[0], mfpGetErrorDescription(data[0]));
+            PrintAndLogEx(ERR, "Sector %u card read error: %02x %s", sectorNo, data[0], mfpGetErrorDescription(data[0]));
             DropField();
             return 5;
         }
         if (datalen != 1 + 16 + 8 + 2) {
-            PrintAndLogEx(ERR, "Sector %d error returned data length:%d", sectorNo, datalen);
+            PrintAndLogEx(ERR, "Sector %u error returned data length:%d", sectorNo, datalen);
             DropField();
             return 6;
         }
@@ -528,24 +530,32 @@ uint8_t mfSectorTrailerOfSector(uint8_t sectorNo) {
 }
 
 // assumes blockno is 0-255..
-uint8_t mfSectorTrailer(uint8_t blockNo) {
+uint8_t mfSectorTrailer(uint16_t blockNo) {
     if (blockNo < 32 * 4) {
         return (blockNo | 0x03);
     } else {
-        return (blockNo | 0x0f);
+        return (blockNo | 0x0F);
     }
 }
 
 // assumes blockno is 0-255..
-bool mfIsSectorTrailer(uint8_t blockNo) {
+bool mfIsSectorTrailer(uint16_t blockNo) {
     return (blockNo == mfSectorTrailer(blockNo));
 }
 
 // assumes blockno is 0-255..
-uint8_t mfSectorNum(uint8_t blockNo) {
+uint8_t mfSectorNum(uint16_t blockNo) {
     if (blockNo < 32 * 4)
-        return blockNo / 4;
+        return (blockNo / 4);
     else
-        return 32 + (blockNo - 32 * 4) / 16;
+        return (32 + (blockNo - 32 * 4) / 16);
 
 }
+
+bool mfIsSectorTrailerBasedOnBlocks(uint8_t sectorno, uint16_t blockno) {
+    if (sectorno < 32) {
+        return ((blockno | 0x03) == blockno);
+    } else {
+        return ((blockno | 0x0F) == blockno);
+    }
+}
