@@ -107,6 +107,7 @@ const static vocabulory_t vocabulory[] = {
     { 0, "data hexsamples" }, 
     { 1, "data hex2bin" }, 
     { 1, "data load" }, 
+    { 1, "data num" }, 
     { 1, "data print" }, 
     { 0, "data samples" }, 
     { 1, "data save" }, 
@@ -297,6 +298,7 @@ const static vocabulory_t vocabulory[] = {
     { 0, "hf legic eload" }, 
     { 0, "hf legic esave" }, 
     { 0, "hf legic eview" }, 
+    { 0, "hf legic einfo" }, 
     { 1, "hf legic crc" }, 
     { 1, "hf legic view" }, 
     { 1, "hf lto help" }, 
@@ -358,21 +360,28 @@ const static vocabulory_t vocabulory[] = {
     { 0, "hf mf gsave" }, 
     { 0, "hf mf gsetblk" }, 
     { 0, "hf mf gview" }, 
+    { 0, "hf mf gdmcfg" }, 
+    { 0, "hf mf gdmsetcfg" }, 
+    { 0, "hf mf gdmsetblk" }, 
     { 0, "hf mf ndefformat" }, 
     { 0, "hf mf ndefread" }, 
     { 0, "hf mf ndefwrite" }, 
     { 1, "hf mfp help" }, 
-    { 0, "hf mfp info" }, 
-    { 0, "hf mfp wrp" }, 
-    { 0, "hf mfp initp" }, 
-    { 0, "hf mfp commitp" }, 
+    { 1, "hf mfp list" }, 
     { 0, "hf mfp auth" }, 
+    { 0, "hf mfp chk" }, 
+    { 0, "hf mfp dump" }, 
+    { 0, "hf mfp info" }, 
+    { 0, "hf mfp mad" }, 
     { 0, "hf mfp rdbl" }, 
     { 0, "hf mfp rdsc" }, 
     { 0, "hf mfp wrbl" }, 
-    { 0, "hf mfp chk" }, 
-    { 0, "hf mfp mad" }, 
+    { 0, "hf mfp commitp" }, 
+    { 0, "hf mfp initp" }, 
+    { 0, "hf mfp wrp" }, 
+    { 0, "hf mfp ndefformat" }, 
     { 0, "hf mfp ndefread" }, 
+    { 0, "hf mfp ndefwrite" }, 
     { 1, "hf mfu help" }, 
     { 1, "hf mfu keygen" }, 
     { 1, "hf mfu pwdgen" }, 
@@ -385,6 +394,7 @@ const static vocabulory_t vocabulory[] = {
     { 0, "hf mfu restore" }, 
     { 1, "hf mfu view" }, 
     { 0, "hf mfu wrbl" }, 
+    { 0, "hf mfu tamper" }, 
     { 0, "hf mfu eload" }, 
     { 0, "hf mfu esave" }, 
     { 0, "hf mfu eview" }, 
@@ -455,14 +465,14 @@ const static vocabulory_t vocabulory[] = {
     { 1, "hf thinfilm list" }, 
     { 0, "hf thinfilm sim" }, 
     { 1, "hf topaz help" }, 
-    { 0, "hf topaz dump" }, 
     { 1, "hf topaz list" }, 
+    { 0, "hf topaz dump" }, 
     { 0, "hf topaz info" }, 
+    { 0, "hf topaz raw" }, 
+    { 0, "hf topaz rdbl" }, 
     { 0, "hf topaz reader" }, 
     { 0, "hf topaz sim" }, 
     { 0, "hf topaz sniff" }, 
-    { 0, "hf topaz raw" }, 
-    { 0, "hf topaz rdbl" }, 
     { 1, "hf topaz view" }, 
     { 0, "hf topaz wrbl" }, 
     { 1, "hf xerox help" }, 
@@ -554,6 +564,7 @@ const static vocabulory_t vocabulory[] = {
     { 0, "lf em 4x50 eview" }, 
     { 0, "lf em 4x50 sim" }, 
     { 1, "lf em 4x70 help" }, 
+    { 0, "lf em 4x70 brute" }, 
     { 0, "lf em 4x70 info" }, 
     { 0, "lf em 4x70 write" }, 
     { 0, "lf em 4x70 unlock" }, 
