@@ -65,8 +65,12 @@ void ProxGuiQT::HideGraphWindow(void) {
 }
 
 // emit picture viewer signals
-void ProxGuiQT::ShowPictureWindow(char *fn) {
-    emit ShowPictureWindowSignal(fn);
+void ProxGuiQT::ShowPictureWindow(const QImage &img) {
+    emit ShowPictureWindowSignal(img);
+}
+
+void ProxGuiQT::ShowBase64PictureWindow(char *b64) {
+    emit ShowBase64PictureWindowSignal(b64);
 }
 
 void ProxGuiQT::RepaintPictureWindow(void) {
@@ -112,24 +116,14 @@ void ProxGuiQT::_HideGraphWindow(void) {
 }
 
 // picture viewer
-void ProxGuiQT::_ShowPictureWindow(char *fn) {
+void ProxGuiQT::_ShowPictureWindow(const QImage &img) {
 
     if (!plotapp)
         return;
 
-    if (fn == NULL)
+    if (img.isNull())
         return;
 
-    size_t slen = strlen(fn);
-    if (slen == 0)
-        return;
-
-    char *myfn = (char *)calloc(slen + 1, sizeof(uint8_t));
-    if (myfn == NULL)
-        return;
-
-    memcpy(myfn, fn, slen);
-
     if (!pictureWidget) {
 
 #if defined(__MACH__) && defined(__APPLE__)
@@ -139,12 +133,7 @@ void ProxGuiQT::_ShowPictureWindow(char *fn) {
         pictureWidget = new PictureWidget();
     }
 
-    QPixmap pm;
-    if (pm.load(myfn) == false) {
-        qWarning("Failed to load %s", myfn);
-    }
-    free(myfn);
-    free(fn);
+    QPixmap pm = QPixmap::fromImage(img);
 
     //QPixmap newPixmap = pm.scaled(QSize(50,50),  Qt::KeepAspectRatio);
     //pm = pm.scaled(pictureController->lbl_pm->size(),  Qt::KeepAspectRatio);
@@ -161,6 +150,53 @@ void ProxGuiQT::_ShowPictureWindow(char *fn) {
     pictureWidget->show();
 }
 
+void ProxGuiQT::_ShowBase64PictureWindow(char *b64) {
+
+    if (!plotapp)
+        return;
+
+    if (b64 == NULL)
+        return;
+
+    size_t slen = strlen(b64);
+    if (slen == 0)
+        return;
+
+    char *myb64data = (char *)calloc(slen + 1, sizeof(uint8_t));
+    if (myb64data == NULL)
+        return;
+
+    memcpy(myb64data, b64, slen);
+
+    if (!pictureWidget) {
+
+#if defined(__MACH__) && defined(__APPLE__)
+        makeFocusable();
+#endif
+
+        pictureWidget = new PictureWidget();
+    }
+
+    QPixmap pm;
+    if (pm.loadFromData(QByteArray::fromBase64(myb64data), "PNG") == false) {
+        qWarning("Failed to read base64 data: %s", myb64data);
+    }
+    free(myb64data);
+    //free(b64);
+
+    pictureController->lbl_pm->setPixmap(pm);
+    pictureController->lbl_pm->setScaledContents(false);
+    pictureController->lbl_pm->setAlignment(Qt::AlignCenter);
+
+    QString s = QString("w: %1  h: %2")
+                .arg(pm.size().width())
+                .arg(pm.size().height()
+                    );
+    pictureController->lbl_sz->setText(s);
+    pictureWidget->show();
+
+}
+
 void ProxGuiQT::_RepaintPictureWindow(void) {
     if (!plotapp || !pictureWidget)
         return;
@@ -213,7 +249,8 @@ void ProxGuiQT::MainLoop() {
     connect(this, SIGNAL(ExitSignal()), this, SLOT(_Exit()));
 
     // hook up picture viewer signals
-    connect(this, SIGNAL(ShowPictureWindowSignal(char *)), this, SLOT(_ShowPictureWindow(char *)));
+    connect(this, SIGNAL(ShowPictureWindowSignal(const QImage &)), this, SLOT(_ShowPictureWindow(const QImage &)));
+    connect(this, SIGNAL(ShowBase64PictureWindowSignal(char *)), this, SLOT(_ShowBase64PictureWindow(char *)));
     connect(this, SIGNAL(RepaintPictureWindowSignal()), this, SLOT(_RepaintPictureWindow()));
     connect(this, SIGNAL(HidePictureWindowSignal()), this, SLOT(_HidePictureWindow()));
 
@@ -560,7 +597,7 @@ void Plot::PlotDemod(uint8_t *buffer, size_t len, QRect plotRect, QRect annotati
             }
             if (j == (int)clk / 2) {
                 //print label
-                sprintf(str, "%u", buffer[i]);
+                snprintf(str, sizeof(str), "%u", buffer[i]);
                 painter->drawText(x - 8, y + ((buffer[i] > 0) ? 18 : -6), str);
             }
         }
@@ -625,11 +662,11 @@ void Plot::PlotGraph(int *buffer, size_t len, QRect plotRect, QRect annotationRe
 
         painter->drawLine(xo - 5, y0, xo + 5, y0);
 
-        sprintf(yLbl, "%d", v);
+        snprintf(yLbl, sizeof(yLbl), "%d", v);
         painter->drawText(xo + 8, y0 + 7, yLbl);
 
         painter->drawLine(xo - 5, y1, xo + 5, y1);
-        sprintf(yLbl, "%d", -v);
+        snprintf(yLbl, sizeof(yLbl), "%d", -v);
         painter->drawText(xo + 8, y1 + 5, yLbl);
         lasty0 = y0;
     }
@@ -637,8 +674,8 @@ void Plot::PlotGraph(int *buffer, size_t len, QRect plotRect, QRect annotationRe
     //Graph annotations
     painter->drawPath(penPath);
     char str[200];
-    sprintf(str, "max=%d  min=%d  mean=%" PRId64 "  n=%u/%zu  CursorAVal=[%d]  CursorBVal=[%d]",
-            vMax, vMin, vMean, g_GraphStop - g_GraphStart, len, buffer[CursorAPos], buffer[CursorBPos]);
+    snprintf(str, sizeof(str), "max=%d  min=%d  mean=%" PRId64 "  n=%u/%zu  CursorAVal=[%d]  CursorBVal=[%d]",
+             vMax, vMin, vMean, g_GraphStop - g_GraphStart, len, buffer[CursorAPos], buffer[CursorBPos]);
     painter->drawText(20, annotationRect.bottom() - 23 - 20 * graphNum, str);
     //clock_t end = clock();
     //double elapsed_secs = double(end - begin) / CLOCKS_PER_SEC;
@@ -752,24 +789,24 @@ void Plot::paintEvent(QPaintEvent *event) {
     char scalestr[30] = {0};
     if (g_CursorScaleFactor != 1) {
         if (g_CursorScaleFactorUnit[0] == '\x00') {
-            sprintf(scalestr, "[%2.2f] ", ((int32_t)(CursorBPos - CursorAPos)) / g_CursorScaleFactor);
+            snprintf(scalestr, sizeof(scalestr), "[%2.2f] ", ((int32_t)(CursorBPos - CursorAPos)) / g_CursorScaleFactor);
         } else {
-            sprintf(scalestr, "[%2.2f %s] ", ((int32_t)(CursorBPos - CursorAPos)) / g_CursorScaleFactor, g_CursorScaleFactorUnit);
+            snprintf(scalestr, sizeof(scalestr), "[%2.2f %s] ", ((int32_t)(CursorBPos - CursorAPos)) / g_CursorScaleFactor, g_CursorScaleFactorUnit);
         }
     }
-    sprintf(str, "@%u..%u  dt=%i %szoom=%2.2f  CursorAPos=%u  CursorBPos=%u  GridX=%lf  GridY=%lf (%s) GridXoffset=%lf",
-            g_GraphStart,
-            g_GraphStop,
-            CursorBPos - CursorAPos,
-            scalestr,
-            g_GraphPixelsPerPoint,
-            CursorAPos,
-            CursorBPos,
-            g_PlotGridXdefault,
-            g_PlotGridYdefault,
-            g_GridLocked ? "Locked" : "Unlocked",
-            g_GridOffset
-           );
+    snprintf(str, sizeof(str), "@%u..%u  dt=%i %szoom=%2.2f  CursorAPos=%u  CursorBPos=%u  GridX=%lf  GridY=%lf (%s) GridXoffset=%lf",
+             g_GraphStart,
+             g_GraphStop,
+             CursorBPos - CursorAPos,
+             scalestr,
+             g_GraphPixelsPerPoint,
+             CursorAPos,
+             CursorBPos,
+             g_PlotGridXdefault,
+             g_PlotGridYdefault,
+             g_GridLocked ? "Locked" : "Unlocked",
+             g_GridOffset
+            );
     painter.setPen(WHITE);
     painter.drawText(20, infoRect.bottom() - 3, str);
 }
