@@ -190,8 +190,10 @@ ISO 7816-4 Basic interindustry commands. For command APDU's.
 #define MIFARE_CMD_RESTORE          0xC2
 #define MIFARE_CMD_TRANSFER         0xB0
 
-#define MIFARE_MAGIC_GDM_AUTH_KEYA  0x80
-#define MIFARE_MAGIC_GDM_AUTH_KEYB  0x81
+#define MIFARE_MAGIC_GDM_AUTH_KEY   0x80
+#define MIFARE_MAGIC_GDM_WRITEBLOCK 0xA8
+#define MIFARE_MAGIC_GDM_READ_CFG   0xE0
+#define MIFARE_MAGIC_GDM_WRITE_CFG  0xE1
 
 #define MIFARE_EV1_PERSONAL_UID     0x40
 #define MIFARE_EV1_SETMODE          0x43
@@ -223,6 +225,9 @@ ISO 7816-4 Basic interindustry commands. For command APDU's.
 #define NTAG_I2C_SELECT_SECTOR      0xC2
 #define NTAG_I2C_FASTWRITE          0xA6
 
+//NTAG 213TT (tamper) command
+#define NTAGTT_CMD_READ_TT          0xA4
+
 // mifare 4bit card answers
 #define CARD_ACK      0x0A  // 1010 - ACK
 #define CARD_NACK_IV  0x00  // 0000 - NACK, invalid argument (invalid page address)
@@ -252,10 +257,12 @@ ISO 7816-4 Basic interindustry commands. For command APDU's.
 #define MAGIC_GEN_1B        2
 #define MAGIC_GEN_2         4
 #define MAGIC_GEN_UNFUSED   5
-#define MAGIC_SUPER         6
-#define MAGIC_NTAG21X       7
-#define MAGIC_GEN_3         8
-#define MAGIC_GEN_4GTU      9
+#define MAGIC_SUPER_GEN1    6
+#define MAGIC_SUPER_GEN2    7
+#define MAGIC_NTAG21X       8
+#define MAGIC_GEN_3         9
+#define MAGIC_GEN_4GTU      10
+#define MAGIC_GEN_4GDM      11
 
 // Commands for configuration of Gen4 GTU cards.
 // see https://github.com/RfidResearchGroup/proxmark3/blob/master/doc/magic_cards_notes.md
